@@ -104,25 +104,26 @@ ret_:
 
 int     mess_y_n(int x1,int y1,char* txtstr)
 {
- int      key;
- int      fcolor_tmp=fColor;
- int      bcolor_tmp=bColor;
- char  newtext[STRSIZ];
+  int      key;
+  int      fcolor_tmp=fColor;
+  int      bcolor_tmp=bColor;
+  char  newtext[STRSIZ];
 
- scrcolor(White,Red);
- sprintf(newtext,"%s\n( Y / N ?) ",txtstr);       
+  scrcolor(White,Red);
+  sprintf(newtext,"%s\n( Y / N ?) ",txtstr);       
 
-   for(;;)
-   {
-      key = message(x1,y1,newtext);
-      switch (key)
-      {
-         case 'N':
-         case 'n': scrcolor(fcolor_tmp,bcolor_tmp);return 0;
-         case 'Y':
-         case 'y': scrcolor(fcolor_tmp,bcolor_tmp);return 1;
-      }
-   }
+  for(;;)
+  {
+    key = message(x1,y1,newtext);
+/*    if(blind) printf(" %s   YES\n",newtext); */
+    switch (key)
+    {
+      case 'N':
+      case 'n': scrcolor(fcolor_tmp,bcolor_tmp);return 0;
+      case 'Y':
+      case 'y': scrcolor(fcolor_tmp,bcolor_tmp);return 1;
+    }
+  }
 }
 
 void  messanykey(int x1,int y1,char* txtstr)
@@ -140,7 +141,6 @@ void  messanykey(int x1,int y1,char* txtstr)
 int informline(long curent, long total)
 {  int  xx;
    int res=0;
-   char  b[51];
    static int Y;
    static int X;
    int xm=where_x(), ym=where_y(), fc=fColor,bc=bColor;
@@ -149,9 +149,13 @@ int informline(long curent, long total)
    {
       Y=maxRow();
       X=15;
-      goto_xy(15,Y); scrcolor(Black,Red);
+/*      goto_xy(15,Y); scrcolor(Black,Red);
       memset(b,' ',50); b[50] = '\0';
-      print("%s",b);
+*/
+     goto_xy(15,Y); scrcolor(White,Red);
+     print(" Calculation in progress. Calculation in progress.");                                                                     
+/*            12345678901234567890123456789012345678901234567890 */
+/*      print("%s",b); */
       goto_xy(15,Y);
       scrcolor(fc,bc);
       escpressed();
@@ -243,7 +247,9 @@ void  menu0(int col,int row,char* label, char* menstr ,
 /* save colors */
    int      fcolor_tmp=fColor;
    int      bcolor_tmp=bColor;
-       
+   void *hscr_=NULL;
+   
+   if(hscr==NULL) hscr=&hscr_;
    lastLine=maxRow();
    if (funcKey == NULL) for (i=0;i<11;i++) fkPos[i]=0; else
    {  int xx;
@@ -270,9 +276,9 @@ void  menu0(int col,int row,char* label, char* menstr ,
       ncol=menstr[0];
       sprintf(fmt,"%%%d.%ds",ncol,ncol);
       height=strlen(menstr)/ncol;
+      if(height==0) { *kk=0; return; }
       if (row+height+1 >lastLine-2) height=lastLine-3-row;
       lastpage = 1+    (strlen(menstr)/ncol -1)/height ;
-       
    if(label[0] ==0 || row == 1) 
    { if (*hscr == NULL)  get_text(col,row,col+ncol+1,row+2,hscr);} 
    else
@@ -305,7 +311,8 @@ label_1:
       scrcolor(box_fg,box_bg);
       chepbox(col,row,col + ncol + 1,row + height + 1);
       scrcolor(star_fg,box_bg);
-      goto_xy(col+1,row); print("*");
+      goto_xy(col+1,row); print("<");
+/*      goto_xy(col+1,row + height+1);  print("?"); */
       scrcolor(page_fg,box_bg);
       if (npage > 1)
       {
@@ -403,16 +410,30 @@ label_4:
 
           break;
 
-		  case KB_RIGHT: /* key_rigth */
-           if (k==lastrow) ink= -81 ;else ink= -80;
-           goto label_4;
+		  case  KB_DOWN: 
+           if(k==lastrow)
+           { 
+              if(npage < lastpage)
+              {  k=1;
+                 npage++;
+                 goto label_1;
+              } else { be_be(); break; }           
+           }else {ink= KB_RIGHT; goto label_4;}
 
+		  case  KB_UP: 
+           if(k==1)
+           {
+              if (npage > 1)
+	      {
+                 k=height;
+		 npage--;
+		 goto label_1;
+              }
+              else{ be_be(); break; }           
+           }
+           else {ink= KB_LEFT; goto label_4;}
 
-		  case KB_LEFT: /* key_left */
-           if (k==1) ink= -73 ; else ink= -72;
-           goto label_4;
-
-		  case KB_UP:   /*  UP */
+		  case KB_LEFT: 
             goto_xy(col1,row + k);
 	    print(fmt,menstr+1+(k-1+(npage-1)*height)*ncol);
             k = k - jump;
@@ -422,8 +443,7 @@ label_4:
 	    print(fmt,menstr+1+(k-1+(npage-1)*height)*ncol);
          break;
 
-
-			case KB_DOWN:   /* DN */
+                  case KB_RIGHT:
 	    goto_xy(col1,row + k);
 	    print(fmt,menstr+1+(k-1+(npage-1)*height)*ncol);
             k = k + jump ;
@@ -433,7 +453,7 @@ label_4:
 	    print(fmt,menstr+1+(k-1+(npage-1)*height)*ncol);
          break;
 
-			case KB_ENTER:    /*  ENTER */
+                  case KB_ENTER:
             scrcolor(box_fg,box_bg);
             chepbox(col,row,col+ncol+1,row+2);
             put_text(&pscr);
@@ -442,15 +462,16 @@ label_4:
 	    print(fmt,menstr+1+(k-1+(npage-1)*height)*ncol);
             *kk = (npage - 1) * height + k;
             goto_xy(1,lastLine);scrcolor(FGmain,BGmain); clr_eol();
+            if(hscr==&hscr_) put_text(hscr); 
             refresh_scr();
             escpressed();
             return;
 
-			case  KB_BACKSP:       /* Backspace */
+			case  KB_BACKSP:
 			case  KB_ESC:
             goto label_3;
 
-			case KB_PAGEU:    /*  PgUp  */
+			case KB_PAGEU:
             if (npage > 1)
             {
                npage--;
@@ -460,7 +481,7 @@ label_4:
                be_be();
          break;
 
-			case KB_PAGED:    /*  PgDn  */
+			case KB_PAGED:
             if (npage < lastpage)
             {
                npage++;
@@ -476,8 +497,8 @@ label_4:
 	case KB_F6: case KB_F7: case KB_F8: case KB_F9: case KB_F10:
 	{  int fk;
            if (funcKey==NULL) break;
-	   if ( ink>=0 && ink <='9') { fk=ink-'0';if (fk==0) fk=10;}
-	      else fk=KB_F1+1-ink;
+	   if ( ink>='0' && ink <='9') { fk=ink-'0';if (fk==0) fk=10;}
+	      else fk=ink-KB_F1+1;
 	   if ((funcKey[fk-1]) != NULL)
            {  void * saveHlp;
 	      get_text(1,lastLine,maxCol(),lastLine,&saveHlp);
@@ -487,10 +508,28 @@ label_4:
 	      (*funcKey[fk-1])((npage-1)*height+k);	 
 	      put_text(&saveHlp);
 	      scrcolor(star_fg,box_bg);
-	      goto_xy(col+1,row); print("*");
+	      goto_xy(col+1,row); print("<");
 	   }
            break;
         }
+        case 6:  /* ^F */
+        case 'f':
+        case 'F':
+        {  char name[32]="";
+           int key=correctStr(5,23,"Enter name(Esc for exit):",name,30,1);
+           if(key)
+           { char * x=strstr(menstr+1,name);
+             if(x==NULL) 
+             {
+               if(blind) sortie(121); else messanykey(10,10, "Not detected"); 
+               break;
+             }
+             k= ((x-menstr)-1)/menstr[0];
+             npage = k/ height + 1;
+             k = k% height + 1;
+             goto label_1;
+           }
+        } break; 
      }
   }
 
@@ -502,6 +541,7 @@ label_3:
    *kk = 0;
 }
 
+
 static char * f_hlp=NULL;
 
 static void f1_key(int x)
@@ -562,7 +602,7 @@ int  str_redact(char* txt,int npos, int maxLen)
    clearTypeAhead();
 
    strcpy(txttmp,txt);
-   if(strlen(txt)>=maxLen) txt[maxLen-1]=0;
+   if(strlen(txt)>maxLen) txt[maxLen]=0;
    first = 1;
    
    scrcolor(t_color,Black);
@@ -722,7 +762,7 @@ static int  correctVar(char* txt,int x,int y, int  type, void * var,int clear)
       err=0;
       if (key == KB_ENTER)
       {
-         trim(buff);
+/*         trim(buff); */
          if(type<0) err=1; else
          {
             switch (type)
@@ -774,3 +814,5 @@ int  correctInt (int x,int y,char* txt,int * var,int clear)
 int  correctStr (int x,int y,char* txt,char * var, int maxLen, int clear)
 {  return correctVar (txt,x,y,-maxLen,var,clear); }
 
+
+
