@@ -1,6 +1,9 @@
 /*
  Copyright (C) 1997, Alexander Kryukov 
+ Corrected by A.Pukhov 
 */
+
+
 /****************************************************************
 
    edittab.c  is a module to view and edit models in CompHEP
@@ -54,6 +57,7 @@ Last revision:	Feb. 01, 1995	horizontal menu
 
 ****************************************************************/
 
+#include <ctype.h>
 #include "syst.h"
 #include "file_scr.h"
 #include "crt.h"
@@ -80,6 +84,9 @@ Last revision:	Feb. 01, 1995	horizontal menu
 #define X2XT(i)  (c_tab.x0+(i)-X1-1)
 #define XT2X(k)  ((k)-c_tab.x0+X1+1)
 
+static void spreadField(int k);
+
+
 typedef struct table0
   {
     table *tab;
@@ -123,38 +130,6 @@ static int kbd_p1=0;      /* out going position in kbd_buff */
 static int kbd_p2=0;      /* in comming position of kbd_buff */
 
 
-/***********************************************************************/
-/*************** Cursor ON/OFF *********************** Jul. 05, 1996 ***/
-/***********************************************************************/
-
-static void drawZCur(char c,int x,int y)
-/*
-   Draw cursor in Zoom mode
-   A.K. Feb, 9, 1998.
-*/
-{
-      scrcolor(White,Black); 
-      goto_xy(x,y);
-      print("%c",c);
-      scrcolor(Black,White);
-      goto_xy(x,y);
-/*      fprintf(stderr,"**** drawZCur(%c,%d,%d)\n",c,x,y);*/
-/**/
-}
-
-static void clearZCur(char c,int x,int y)
-/*
-   Clear cursor in Zoom mode
-   A.K. Feb, 9, 1998.
-*/
-{
-      scrcolor(Black,White);
-      goto_xy(x,y);
-      print("%c",c);
-      goto_xy(x,y);
-/*      fprintf(stderr,"**** clearZCur(%c,%d,%d)\n",c,x,y);*/
-/**/
-}
 
 /***********************************************************************/
 /*************** Horizontal button bar *************** Jan. 31, 1995 ***/
@@ -270,7 +245,7 @@ typedef struct hbl_struct {
 static hbl_struct *hbl_add(hbl_struct * hbl,hbb_struct * hbb) {
   hbl_struct *new;
   new=(hbl_struct *)m_alloc(sizeof(hbl_struct));
-  if (!new) exit(99);
+  if (!new) sortie(59);
   new->hbb=hbb;
   new->next=hbl;
   return new;
@@ -315,21 +290,20 @@ static void hbl_colors(hbl_struct * hbl,int fgc,int bgc,int fgc_off,int bgc_off)
 
 static hbb_item up_items[6]={
    {"Clr",CTRL('C'),1},
-   {"Rest",CTRL('R'),1},
    {"Del",CTRL('D'),1},
    {"Size",CTRL('S'),1},
+   {"Read",CTRL('R'),1},
+   {"ErrMes",CTRL('E'),1},
    {"",0,0}
 };
 
-static hbb_item down_items[9]={
+static hbb_item down_items[7]={
    {"F1",KB_F1,1},
    {"F2",KB_F2,1},
-   {"Top",20,1},
-   {"Bottom",2 ,1},
-   {"GoTo",CTRL('G'),1},
+   {"Xgoto",CTRL('X'),1},
+   {"Ygoto",CTRL('Y'),1},
    {"Find",CTRL('F'),1},
-   {"Zoom",CTRL('Z'),1},
-   {"ErrMes",CTRL('E'),1},
+   {"Write",CTRL('W'),1},
    {"",0,0}
 };
 
@@ -351,7 +325,7 @@ static void push_key(int kk){
   kbd_buff[kbd_p2]=kk;
   kbd_p2++;
   if (kbd_p2==KBD_BUFF_SIZE) kbd_p2=0;
-  if (kbd_p2 == kbd_p1) exit(99);
+  if (kbd_p2 == kbd_p1) sortie(59);
 }
 
 static int pop_key(void) {
@@ -402,7 +376,7 @@ static void redrawTable(table * tab) {
      { 
        strcpy(buffer,lastline->line+c_tab.x0-1);
        buffer[c_tab.width]=0;
-       print(buffer);
+       print("%s",buffer);
        lastline=lastline->next;
      }
      print("%*.*s",X2-where_x(),X2-where_x(),"");
@@ -443,7 +417,7 @@ static int shiftY(int y) {
 
 static void  setFieldFrame( int xPos, char * format )
 { int z;
-   for(z=xPos-X1+c_tab.x0-1; z<x2t && format[z]!=SEPR_CHAR;z++);
+   for(z=xPos-X1+c_tab.x0-2; z<x2t && format[z]!=SEPR_CHAR;z++);
    c_tab.x2= z+X1-c_tab.x0+1;
 
    for(z= xPos -X1+c_tab.x0-3; z >=0 && format[z]!=SEPR_CHAR; z--);
@@ -500,7 +474,7 @@ static int s_edit(int x1,  int x2, int x0, int  y1, char * s, int show )
   int i;
   int rc = 0, rc1 = 0;
   int key = 0;
-  int len,z,slen0=strlen(s);
+  int len,z;
   int dx,dy;
 
   len=x2 - x1+1;
@@ -566,14 +540,15 @@ static int s_edit(int x1,  int x2, int x0, int  y1, char * s, int show )
 
       case KB_PAGEU :
       case KB_PAGED:
-      case CTRL('B'):
       case CTRL('D'):
-      case CTRL('T'):
       case CTRL('E'):
       case CTRL('F'):
-      case CTRL('G'):
+      case CTRL('Y'):
+      case CTRL('X'):
       case CTRL('S'):
       case CTRL('Z'):     
+      case CTRL('R'): 
+      case CTRL('W'):
       case KB_F1:
       case KB_F2:             
         push_key(TEMPORAL);
@@ -581,12 +556,6 @@ static int s_edit(int x1,  int x2, int x0, int  y1, char * s, int show )
         break; 
 
 
-      case CTRL('R'):
-       rc1=0;
-       push_key(TEMPORAL);
-       push_key(key);
-       break;
-
       case KB_MOUSE:
           if (mouse_info.but1 == MRELEASED ) 
 	  { 
@@ -610,25 +579,40 @@ static int s_edit(int x1,  int x2, int x0, int  y1, char * s, int show )
         break;
 
      default:
-		if ((key>=32)&&(key<=127)) {
-			if (s[strlen(s)-1] == ' ' || strlen(s) < slen0) {
-				for(i=strlen(s)-2; i >= x0+scrCur-x1; i--) s[i+1] = s[i];
-				s[x0+scrCur-x1] = (char) key;
-				/*if (scrCur<x2)*/ scrCur++;
-				/*else {if (s[x0+scrCur-x1+1]) x0++;}*/
-			} else be_be();
-			rc1=1;
-		}
+        if((key>=32)&&(key<=127)) 
+        {
+/*
+           if(s[strlen(s)-1] != ' ' && strlen(s) == slen0)
+           {
+           printf("Try spreadField(4)\n");
+                spreadField(4);   redrawTable( c_tab.tab);        
+           
+           }
+*/
+           if(s[x0+scrCur-x1]==0 ||s[strlen(s)-1] != ' ')
+           {
+              push_key(TEMPORAL); push_key(-150);  push_key(key);               
+           
+           } 
+           else 
+           {
+              for(i=strlen(s)-2; i >= x0+scrCur-x1; i--) s[i+1] = s[i];
+	      s[x0+scrCur-x1] =  key;
+              scrCur++;
+           }
+           rc1=1;
+	}
         key = 0;
 	break;
     } /* switch */
-      
-         if(scrCur>x2) { push_key(TEMPORAL); push_key(KB_RIGHT); push_key(1);}  
-    else if(scrCur<x1) { push_key(TEMPORAL); push_key(KB_LEFT);  push_key(1);}
+    if(scrCur>=X2||scrCur>x2+1) { push_key(TEMPORAL); push_key(KB_RIGHT); push_key(0);}
+    else if(scrCur<x1)  { push_key(TEMPORAL); push_key(KB_LEFT);  push_key(0);}
     else
     { 
       scrcolor(White,Blue); goto_xy(x1,y1); print("%.*s",len,s+x0);
-      scrcolor(Blue,Yellow);goto_xy(scrCur,y1);  print("%c",s[x0+scrCur-x1]);   
+ if(s[x0+len]==0 &&x1+len+1<X2    ){ scrcolor(Black,White);  print("|");}
+      scrcolor(Blue,Yellow);goto_xy(scrCur,y1);
+ if(s[x0+scrCur-x1]==0) print("|"); else print("%c",s[x0+scrCur-x1]);   
     }
     goto_xy(scrCur,y1);
 
@@ -639,341 +623,10 @@ static int s_edit(int x1,  int x2, int x0, int  y1, char * s, int show )
   if (key == KB_ESC){push_key(KB_ESC); /* rc1 = 0;*/} /*  A.K.   Feb. 9, 1998 */
 
   scrcolor(Black,White); goto_xy(x1,y1); print("%.*s",len,s+x0);
+  if(x1+len+1<X2 && s[x0+len]==0)  print("|");
   return rc1;
 } /* s_edit */
 
-/****************** Zoom *******************/
-
-static int wZoom;/* Width of zoom window without border */
-static int hZoom;/* High of zoom window without border */
-static int sLen; /* The length of the zoomed string */
-static int sEnd; /* The real end of the zoomed string */
-
-static int xy2z(int x,int y,int * z){
-/*                                        May 12, 1994
-   xy2z transorm 2-dim coordinate to linear coordinate along
-	the working string.
-
-   x,y - 2-dim. coordinate. Start point is (0,0). Last on is 
-	 (hZoom-1,wZoom-1).
-   *z   - linear coordinate. Start point is 0. Last one is sLen-1.
-
-   return - 0 if ok and 1 if tranformation impossible.
-*/
-   int z1;        /* Work vars. */
-
-   z1 = wZoom*y+x;
-   if (z1<0) return -1;
-   else if (z1>=sLen) return 1;
-   else {
-     *z = z1;
-     return 0;
-   };
-}
-
-static int z2xy(int z,int * x,int * y){
-/*                                        May 12, 1994
-   z2xy transorm linear coordinate along the working string to 
-	2-dim. coordinate.
-
-   *x,*y - 2-dim. coordinate. Start point is (0,0). Last on is 
-	 (hZoom-1,wZoom-1).
-   z   - linear coordinate. Start point is 0. Last one is sLen-1.
-
-   return - 0 if ok and 1 if tranformation impossible.
-*/
-   int x1,y1;         /* Work vars. */
-
-   y1 = z/wZoom;
-   x1 = z - (wZoom*y1);
-   /*printf("*** z->(x,y)=%d->(%d,%d)",z,*x,*y); inkey();*/
-   if ((x1<0) || (x1>=wZoom) || (y1<0) || (y1>=hZoom)) return 1;
-   else {
-     *x=x1;
-     *y=y1;
-     return 0;
-   };
-}
-
-
-static void drawLStr(int x,int y,char * s,int z){
-/*
-   drawLStr draw logn string in the window prepare before.
-   x,y - the start point on the screen.
-   s   - drawable string.
-   z   - number of intial char. from s.
-*/
-  int i;        /* cycle counter */
-  int x1=0,y1=0;/* local coordinate */
-  char ws[STRSIZ];  /* work string */
-
-  if (z>=sEnd) sEnd=z+1;
-  if (z2xy(z,&x1,&y1)) exit(99);
-  z=y1*wZoom;
-  for(;y1<hZoom && z<sEnd+1;y1++){
-    for(i=x1;(i<wZoom)&&s[z+i]&&z+i<sEnd+1;i++) 
-    ws[i]=s[y1*wZoom+i];
-    ws[i]=0;
-    goto_xy(x+x1,y+y1);
-    print(&(ws[x1]));
-    x1=0;
-    z+=wZoom;
-  };
-}
-
-
-static void clZWin(int x,int y){
-/*
-   clZein clear window for zoom.
-   x,y - the start point on the screen.
-   s   - drawable string.
-   z   - number of intial char. from s.
-*/
-  int i;        /* cycle counter */
-  int y1=0;     /* local coordinate */
-  char ws[STRSIZ];  /* work string */
-
-  for(;y1<hZoom;y1++){
-    for(i=0;(i<wZoom)&&(y1*wZoom+i)<sLen;i++) ws[i]=' ';
-    if (y1==(hZoom-1)) for(;i<wZoom;i++) ws[i]='<';
-    ws[i]=0;
-    goto_xy(x,y+y1);
-    print(ws);
-  };
-}
-
-static int insChar(char c,char * s,int z){
-  int i;           /* Cycle var. */
-  int l;           /* length of str. */
-
-  l = strlen(s);
-  if (s[l-1]!=' ') return 1;
-  for (i=l-2;i>=z;i--) s[i+1]=s[i];
-  s[z]=c;
-  sEnd++;
-  return 0;
-}
-
-static int delChar(char * s,int z){
-  int i;           /* Cycle var. */
-  int l;           /* length of str. */
-
-  l = strlen(s);
-  for (i=z+1;i<l;i++) s[i-1]=s[i];
-  s[l-1]=' ';
-  sEnd--;
-  return 0;
-}
-
-static int zoomStr(char * ws,int x,int y,int b,int show){
-/*                                      May 12, 1994 
-   zoom shows long fields in separate window.
-
-   ws - zoomed string
-   x,y - the coordinates of left-top angle
-   b - the width of border;
-   show - showed flag.
-   return - 0
-
-   Modifications:
-   A.K. Feb, 9, 1998.      clear and draw cursor.
-   A.K. Feb. 9, 1998       restore old string if press [esc]
-*/
-void * pbuff;      /* pointer to saving buffer */
-int key;            /* code of pressed key */
-int x1,y1;          /* shifted screen coordinate. the first position 
-		       of the text */
-int edit=0;         /* Is it edited? Not implemented */
-int zc,zc1;         /* Linear curses coordinate. zc1-previous pos.*/
-int xc,yc,xc1,yc1;  /* Local curses coordinate. xc1,yc1 - previous pos. */
-int dx,dy;
-int rc;
-
-wZoom = 75;
-if (strlen(ws)>=STRSIZE) exit(99);
-if (show) trim(ws);
-sLen  = strlen(ws);
-for (sEnd=sLen-1;sEnd && ws[sEnd]==' ';sEnd--){};
-sEnd++;
-hZoom = STRSIZE/wZoom+1;
-z2xy(sLen-1,&xc,&yc);
-hZoom = yc;
-if (!hZoom) wZoom=xc+1; 
-   /*printf("*** (x,y)=(%d,%d)",x,y); inkey();*/
-hZoom++;
-x1=x+b;
-y1=y+b;
-zc=0;
-z2xy(zc,&xc,&yc);
-
-/*get_text(x,y,x+wZoom-1+2*b,y+hZoom-1+2*b,&pbuff);     ????*/
-get_text(x,y,x+wZoom-1+2*b,y+hZoom+2*b,&pbuff);
-/*printf(" x=%d,y=%d,wZoom=%d,hZoom=%d,b=%d\n",x,y,wZoom,hZoom,b);*/
-scrcolor(Black,White);
-if (b>0) chepbox(x,y,x+wZoom-1+2*b,y+hZoom-1+2*b);
-
-key = 0;
-clZWin(x1,y1);
-
-for(;key!=KB_ESC && key!=KB_ENTER;){
-  zc1=zc;
-  switch (key){
-  case 0:
-    drawLStr(x1,y1,ws,0);
-    drawZCur(ws[0],x1+xc,y1+yc);
-    goto_xy(x1+xc,y1+yc);
-    break;
-
-  case KB_LEFT:
-    dx=pop_key();
-    for(;dx>0 && zc>0;dx--) {
-      zc--;
-      z2xy(zc,&xc,&yc);
-      goto_xy(x1+xc,y1+yc);
-    };
-    break;
-
-  case KB_RIGHT:
-    dx=pop_key();
-    for(;dx>0 /*&& zc<sLen-1*/;dx--) {
-      zc++; if(zc==sLen-1) zc=0;
-            z2xy(zc,&xc,&yc);
-      goto_xy(x1+xc,y1+yc);
-    };
-    break;
-
-  case KB_UP:
-    dy=pop_key();
-    for(;dy>0 && yc>0;dy--) {
-      yc--;
-      xy2z(xc,yc,&zc);
-      goto_xy(x1+xc,y1+yc);
-    };
-    break;
-
-  case KB_DOWN:
-    dy=pop_key();
-    for(;dy>0 && yc<hZoom-1;dy--) {
-      yc++;
-      if (xy2z(xc,yc,&zc)) yc--;
-      goto_xy(x1+xc,y1+yc);
-    };
-    break;
-
-  case KB_PAGEU:
-  case KB_HOME:
-    zc=0;
-    z2xy(zc,&xc,&yc);
-    goto_xy(x1+xc,y1+yc);
-    break;
-
-  case KB_PAGED:
-  case KB_END:
-    zc=sLen-1;
-    z2xy(zc,&xc,&yc);
-    goto_xy(x1+xc,y1+yc);
-    break;
-
-  case KB_DC:    /* Del. char. */
-    if (show) break;
-    if(!delChar(ws,zc)) edit=1;
-    drawLStr(x1,y1,ws,zc);
-    goto_xy(x1+xc,y1+yc);
-    break;
-
-  case KB_BACKSP:    /* Del. char. before */
-    if (show) break;
-    if (zc<=0) break;
-    zc--;
-    z2xy(zc,&xc,&yc);
-    goto_xy(x1+xc,y1+yc);
-    if(!delChar(ws,zc)) edit=1;
-    drawLStr(x1,y1,ws,zc);
-    goto_xy(x1+xc,y1+yc);
-    break;
-
-      case KB_MOUSE:
-          if (mouse_info.but1 == MRELEASED ) { 
-#if EDITTAB_DEBUG
-            printf(" mouse: but1=%d, x=%d, y=%d\n",
-                 mouse_info.but1, mouse_info.row,
-                 mouse_info.col );
-            printf(" Z-cursor: xc=%d, yc=%d, zc=%d\n",xc,yc,zc);
-#endif
-            if (rc=hbl_select(hbl->next)) {
-               push_key(rc);
-               if (rc==KB_UP) push_key(1);
-               else if (rc==KB_DOWN) push_key(1);
-               else if (rc==KB_LEFT) push_key(1);
-               else if (rc==KB_RIGHT) push_key(1);
-            }
-            else if (m_inbox(x,y,x+wZoom-1+2*b,y+hZoom-1+2*b)) {
-              dx=mouse_info.col-x1-xc;
-              if (dx>0) { 
-                push_key(KB_RIGHT); 
-                push_key(dx);
-              }
-              else if (dx<0) { 
-                push_key(KB_LEFT); 
-                push_key(-dx);
-              };
-              dy=mouse_info.row-y1-yc;
-              if (dy>0) { 
-                push_key(KB_DOWN); 
-                push_key(dy);
-              }
-              else if (dy<0) { 
-                push_key(KB_UP); 
-                push_key(-dy);
-              };
-            };
-          };
-        break;
-
-	 case KB_F1:
-	     show_help(ZE_HLP);
-	     scrcolor(Black,White);
-	 break;
-
-	 case KB_F2:
-	     show_help("zoomed");
-	     scrcolor(Black,White);
-	 break;
-
-  default:
-    if (show) break;
-    if ((key>=32)&&(key<=127)) {
-      if (insChar((char)key,ws,zc)) break;
-      edit=1;
-      drawLStr(x1,y1,ws,zc);
-      if(zc<sLen-1)zc++;
-      z2xy(zc,&xc,&yc);
-      goto_xy(x1+xc,y1+yc);
-    };
-    break;
-  }; /* switch (key) */
-  
-  if (!show) {
-    goto_xy(x+1,y);
-    scrcolor(White,Black);
-    print("%d%c%c%c",zc+1,boxFrame[5],boxFrame[5],boxFrame[5]);
-    goto_xy(x1+xc,y1+yc);
-    drawZCur(ws[zc],x1+xc,y1+yc);
-    if(zc!=zc1){
-      z2xy(zc1,&xc1,&yc1);
-      clearZCur(ws[zc1],x1+xc1,y1+yc1);
-    };
-    scrcolor(Black,White);
-  }
-  key = pop_key();
-}; /* while */
-
-if (key==KB_ESC) edit=0;  /* A.K. Feb. 9, 1998 */
-
-scrcolor(White,Black);
-put_text(&pbuff);
-return edit;
-}
 /*************** Resize Table *************/
 
 static void squeezeField(int k) {
@@ -1129,10 +782,11 @@ int  edittable(int xx,int yy,table * tab,int ntop,char * hlpf,int show)
   void *   pscr;
   int   i,l;
   char      ws[STRSIZE];
-  int   edit=FALSE;
+  int   edit=0;
   int dx,dy;   /*  Mouse */
   int rc;
   
+  if(!blind && tab->pos) ntop=tab->pos;  
   clearTypeAhead();
   initEdit(tab,xx,yy, ntop);
   e_box.x1=X1;
@@ -1155,7 +809,7 @@ int  edittable(int xx,int yy,table * tab,int ntop,char * hlpf,int show)
      }
   }
   scrcolor(Red,Blue);
-  goto_xy(X1,Y1-1);print("%s","*");
+  goto_xy(X1,Y1-1);print("*");
   /* up_bottom bar */
   hbb_new(&up_bar,&up_items[0],boxFrame[1]);
   hbb_place(&up_bar,1+X1,Y1);
@@ -1181,9 +835,9 @@ int  edittable(int xx,int yy,table * tab,int ntop,char * hlpf,int show)
        if(s_edit(MAX(X1+1,c_tab.x1),MIN(X2-1,c_tab.x2),
                                     MAX(0,X1-c_tab.x1+1),
 	      			     c_tab.y,ws,show) ) 
-       {
-          edit=TRUE;
-          for(i=0; ws[i]; i++) c_tab.edit->line[c_tab.x0+c_tab.x1 +i-X1-2]=ws[i];
+       {  char *c=(c_tab.edit->line) +c_tab.x0+c_tab.x1 -X1-2;
+          for(i=0; ws[i]; i++) c[i]=ws[i];
+          edit=1;
        }
 
     }
@@ -1280,8 +934,11 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	   } /* if */
     	   c_tab.del=del2;								           
     	}										           
+    	edit=1;
+        scrCur  = X1-c_tab.x0+2;						                   
+        setFieldFrame(scrCur,tab->format);						                   
+        if(scrCur<=X1 ||scrCur>=X2 )shiftL(X1-scrCur+1,tab);				                   
     	redrawTable(tab);
-    	edit=TRUE;
     	push_key(KB_DOWN);
         push_key(1);									                   
     	break;
@@ -1305,20 +962,40 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	}
         break;
 										                   
-    case KB_TAB:									        
+    case KB_TAB:   /* by A.Pukhov */									        
         for(;TABCUR<x2t&&tab->format[TABCUR]!=SEPR_CHAR;scrCur++);			        
         scrCur++;									                   
         if(TABCUR>=x2t) scrCur -=TABCUR;						                   
         setFieldFrame(scrCur,tab->format);						                   
         if(scrCur<=X1 ||scrCur>=X2 )shiftL(X1-scrCur+1,tab);				                   
         break;										                   
+
+    case  CTRL('X'):  /* by A.Pukhov */
+       
+    	{  int nn=1,i;
+    	   int scrCur_old=scrCur;
+    	   						    
+    	   if(!correctInt(X1+1,Y2,"  Go to: ",&nn,1)) break;		    
+           for(;TABCUR>=0 && tab->format[TABCUR]!=SEPR_CHAR;scrCur--);
+           scrCur++;
+           for(i=1;i<nn && TABCUR<x2t && tab->format[TABCUR]!=SEPR_CHAR; scrCur++,i++);
+           if(i<nn)
+           { messanykey(10,5,"Wrong position");
+             scrCur=scrCur_old;
+             break;
+           }
+           setFieldFrame(scrCur,tab->format);
+           if(scrCur<=X1 ||scrCur>=X2 )shiftL(X1-scrCur+1,tab);
+        }
+        break;
+                                                    
     											        
-    case KB_RIGHT:									              
+    case KB_RIGHT:								              
     	dx=pop_key(); /* suppose  dx==1 */
-        if(TABCUR>=x2t) scrCur--; else							        			                   
-        {										                   
-           if(tab->format[TABCUR]==SEPR_CHAR)						                   
-           {  scrCur++;									                   
+        if(TABCUR>x2t) scrCur--;else							        			                   
+        {		
+/*           if(tab->format[TABCUR]==SEPR_CHAR)	*/					                   
+           {  scrCur+=dx;									                   
               setFieldFrame(scrCur,tab->format);					                   
            }										                   
            if(scrCur>=X2) shiftL(-20,tab);						                   
@@ -1329,24 +1006,14 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	dx=pop_key(); /* suppose  dx==1 */
         if(TABCUR<0) scrCur++;else							        		                   
         {										                   
-           if(tab->format[TABCUR]==SEPR_CHAR)						                   
-           {  scrCur--;									                   
+/*           if(tab->format[TABCUR]==SEPR_CHAR)	*/					                   
+           {  scrCur-=dx;									                   
               setFieldFrame(scrCur,tab->format);					                   
            }										                   
            if(scrCur<=X1)  shiftL(20,tab);						                   
         }										                   
     	break;
 
-    case CTRL('B'):     /* Bottom    */							        
-    	for (i=Y1+2;c_tab.edit->next!=NULL;i++)
-        {										        
-    	  c_tab.top=c_tab.top->next;
-    	  c_tab.y0++;
-    	  c_tab.edit=c_tab.edit->next;
-        }										                   
-    	redrawTable(tab);
-    	break;
-
     case CTRL('D'):									        
     	if (show) {key=0;break;}
     	if (c_tab.del) free(c_tab.del);
@@ -1369,7 +1036,7 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	   c_tab.edit = c_tab.top;
     	}
     	else
-        {										        
+        {   
     	   if (c_tab.edit->next)
            {										        
     	      c_tab.edit = c_tab.edit->next;
@@ -1386,23 +1053,112 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	c_tab.del->next = NULL;
     	c_tab.del->pred = NULL;
     	redrawTable(tab);
-    	edit=TRUE;
+    	edit=1;
     	break;
 
-    case CTRL('F'):									        
+    case CTRL('R'):  /*  by A.Pukhov */
+        if(!show)
+        { char  fname[20]="";
+          FILE*f;
+          int size=c_tab.x2 - c_tab.x1 +1;
+          char buff[STRSIZ], *s;
+          int i,c,k;                                     
+          correctStr(X1+1,Y2,"File name:",fname,15,1);
+          trim(fname);
+          f=fopen(fname,"r");
+          if(!f)  { messanykey(10,5,"Wrong file name"); break;}
+          for(k=0;k<STRSIZ;)
+          {  c=fgetc(f); 
+             if(c==EOF) break;
+             if(isspace(c)) continue;
+             buff[k++]=c;
+          }
+          fclose(f);
+          if(k>size)
+          { 
+//            if(c_tab.tab->format[X2XT(c_tab.x2)-1] == RESIZE_CHAR)
+            {  int min,max;
+                minmaxFSize(&min,&max);
+                if(k<=max) {spreadField(k-size); size=c_tab.x2 - c_tab.x1 +1;}
+            }
+            if(k>size) {messanykey(10,5,"Too long record"); break;}
+          }
+          s=c_tab.edit->line+c_tab.x0+c_tab.x1-X1-2;
+          for(i=0;i<size  ;i++) if(i<k) s[i]=buff[i]; else s[i]=' ';
+          redrawTable(tab);
+          edit=1;           
+        }
+        break;
+
+    case CTRL('W'):    /* by A.Pukhov */
+        { char  fname[20]="";
+          FILE*f;
+          char *s;
+          int size=c_tab.x2 - c_tab.x1 +1;
+          int i,c,k;                                     
+          correctStr(X1+1,Y2,"File name:",fname,15,1);
+          trim(fname);
+          f=fopen(fname,"w");
+          if(!f)  { messanykey(10,5,"Wrong file name"); break;}
+          s=c_tab.edit->line+c_tab.x0+c_tab.x1-X1-2;
+          
+          for(k=0,i=0;i<size;i++,k++)
+          { c=s[i];
+            if(isspace(c)) continue;
+            fputc(c,f);
+            if(k>70 && strchr("+-*/)",c)) { fputc('\n',f);k=0;}
+          }
+          fputc('\n',f);
+          fclose(f);
+        }  
+        break;
+
+
+    case CTRL('F'):  /* by A.Pukhov */									        
     	{  static linelist lastLine=NULL;
            static int      lastPos =-1;							              	
     	   linelist nn=c_tab.edit;
            if(  (lastLine==c_tab.edit && lastPos==TABCUR)				        
               || correctStr(X1+1,Y2," Find:",fLine,15,1) )				        
-    	   {  int dy=0;
-              char *ip=strstr(nn->line+TABCUR,fLine);					                           
-              if(!ip)									        
-              { nn=nn->next; dy++;							                           
-                for(;nn&&!(ip=strstr(nn->line,fLine));nn=nn->next) dy++;		                           
-              }									                           
+    	   {  int i, dy=0, shift=TABCUR;
+    	      char * f[6]={fLine,NULL,NULL,NULL,NULL,NULL};
+    	      char * p[6]={NULL, NULL,NULL,NULL,NULL,NULL};
+    	      int    d[6]={0,0,0,0,0,0};
+    	      char *ip=NULL;
+
+              for(i=0;(i<5)&&f[i];i++)
+              {  f[i+1]=strchr(f[i],',');
+                 if(f[i+1]) {f[i+1][0]=0; f[i+1]++;}
+              }
+              
+              if(f[5]) 
+              { messanykey(10,5,"Too many  items for search");
+                for(i=1;f[i]&&(i<6);i++) f[i][-1]=',';
+                break;
+              } 
+
+              for(i--;i>0;i--)
+              { int j;
+                for(j=i-1;j>=0;j--) if(strcmp(f[i],f[j])==0) {d[i]=i-j; break;}
+              }
+
+              for(nn=c_tab.edit; nn ; nn=nn->next,dy++,shift=0)
+              { p[0]=strstr(nn->line+shift,f[0]);
+                if(!(*p)) continue;
+                p[0]+=strlen(f[0]);
+                ip=p[0];                                                                                                            
+                for(i=1; ip&&f[i]; i++)
+                { 
+                  if(d[i]) p[i]=strstr(p[i-d[i]],f[i]);
+                  else     p[i]=strstr(nn->line+shift,f[i]);
+                  if(!p[i]) ip=NULL;   else {p[i]+=strlen(f[i]);  
+                  if(p[i]<ip) ip=p[i];}
+                }
+                if(ip) break;
+              }
+
               if(ip)									                   
-              {  scrCur+=((ip - nn->line)-TABCUR+strlen(fLine));			        
+              {  scrCur+=((ip - nn->line)-TABCUR/*+strlen(fLine)*/);			        
                  if(TABCUR>=x2t) scrCur--;						                           
                  setFieldFrame(scrCur,tab->format);					                           
                  if(scrCur<=X1||scrCur>=X2) shiftL(X1+strlen(fLine)+1-scrCur,tab);	                           
@@ -1414,19 +1170,25 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
                  lastLine=NULL;								        
                  lastPos=-1;								        
                  messanykey(10,5,"String not found");					        
-              }									        
+              }	
+              for(i=1;f[i];i++) f[i][-1]=',';								        
     	   }
     	}
     	break;
 
-    case CTRL('G'):									        
-    	{  int nn=1;						    
-    	   if(correctInt(X1+1,Y2,"  Go to: ",&nn,1))		    
-    	   {							    
-    		 dy=nn-(c_tab.y0+ (c_tab.y-Y1-2 ));
-    		 if (dy>0)       {push_key(KB_DOWN);push_key(dy);}
-    	      else if (dy<0) {push_key(KB_UP);push_key(-dy);}	    
-    	   }							    
+    case CTRL('Y'):									        
+    	{ int nn=1;
+    	  char sss[8]="1      ";
+    	  correctStr(X1+1,Y2," GoTo:",sss,7,1);
+    	  if(sscanf(sss,"%d",&nn)==1)						    		    
+    	  {							    
+            dy=nn-(c_tab.y0+ (c_tab.y-Y1-2 ));
+    	    if (dy>0)      {push_key(KB_DOWN);push_key(dy);}
+    	    else if (dy<0) {push_key(KB_UP);push_key(-dy);}	    
+          }else if(strchr(sss,'$'))
+          { for(;c_tab.edit->next; c_tab.top=c_tab.top->next,c_tab.y0++,c_tab.edit=c_tab.edit->next);
+             redrawTable(tab);
+          }
     	}							    
     	break;							    
 
@@ -1434,30 +1196,33 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
     	if(errorText[0]) messanykey(10,10,errorText);else be_be();
     	 break;
 
-    case CTRL('T'):     /*  Top    */							                           
-        push_key(KB_UP);								                   
-        push_key( c_tab.y0+ c_tab.y-Y1-3 );						                   
-    	break;
-
     case CTRL('S'):									        
     	if (show)  break;
-    	if(c_tab.tab->format[X2XT(c_tab.x2)-1] == RESIZE_CHAR
-          &&	resizeField()) {edit = 1; redrawTable(tab);}				        				      	
-    	break;
-
-    case CTRL('Z'):									        
-    	strcpy( ws,&c_tab.edit->line[c_tab.x0+c_tab.x1-X1-2]);
-    	ws[c_tab.x2 - c_tab.x1 + 1] = 0;
-        hbl_close(hbl);									                   
-    	if (zoomStr(ws,3,3,1,show))
-    	{  edit=1;
-    	   for(i=0; ws[i]; i++)
-    		 c_tab.edit->line[c_tab.x0+c_tab.x1 +i-X1-2]=ws[i];
-
-    	};
-    	hbl_open(hbl);
+    	if(
+//    	c_tab.tab->format[X2XT(c_tab.x2)-1] == RESIZE_CHAR &&	
+    	resizeField()) {edit = 1; redrawTable(tab);}				        				      	
     	break;
+    	
+    case -150:   /* automatic size inclease by A.Pukhov */
+        dx=pop_key();
+        if(show) break;
+//        if(c_tab.tab->format[X2XT(c_tab.x2)-1] == RESIZE_CHAR)
+        {  int d,min,max; 
+           int kk=c_tab.x2-c_tab.x1+1;
+           
+           minmaxFSize(&min,&max);
 
+           d=(max-kk<4)? max-kk:4;
+           if(d)
+           {  spreadField(d);
+              redrawTable(tab);
+              edit = 1;
+              push_key(dx);
+           } else be_be();
+        }
+//         else be_be();  
+        break;
+        
     case KB_F1: show_help(hlpf);      break;						        
     case KB_F2: show_help("edittab"); break;						        
 
@@ -1475,5 +1240,6 @@ printf(" cursor: x=%d, x1=%d, y=%d\n",c_tab.x1,c_tab.x2,c_tab.y);
   if (hbl) { hbl_close(hbl); hbl_delete(hbl);};
   scrcolor(White,Black);
   put_text(&pscr);
+  tab->pos=c_tab.y0+ (c_tab.y-Y1-2);
   return edit;
 } 
