@@ -785,7 +785,8 @@ double pWidth(char *name, txtList * LL)
   for(i=0;i<nModelParticles;i++)
   { char *pnames[2]={ModelPrtcls[i].name,ModelPrtcls[i].aname};
     for(j=0;j<2;j++) if(strcmp(name,pnames[j])==0) 
-    { if(decayTable[i].status==1)
+    { 
+      if(decayTable[i].status==1)
       { 
         if(LL) *LL=decayTable[i].pdList[j];
         return decayTable[i].width;
@@ -838,7 +839,7 @@ double pWidth(char *name, txtList * LL)
         }
      }          
   }
-  
+  decayTable[i0].status=-1;
   if(Q==NULL) for(i=0;i<nModelVars;i++) if(strcmp(varNames[i],"Q")==0){ Q= varValues+i; break;}
   if(Q) { Qstat=*Q; setQforParticle(Q,name);}
     
@@ -851,6 +852,7 @@ double pWidth(char *name, txtList * LL)
     if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname)) 
                  decayTable[i0].pdList[1-j0]=conBrList(L); 
     decayTable[i0].width=width;
+    decayTable[i0].status=1;
     if(Q) {*Q=Qstat; calcMainFunc();}
     return width;
   }
@@ -895,6 +897,7 @@ double pWidth(char *name, txtList * LL)
   if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname)) 
                decayTable[i0].pdList[1-j0]=conBrList(Lout);
   decayTable[i0].width=sum;
+  decayTable[i0].status=1;
   if(Q) { *Q=Qstat; calcMainFunc();}
   return sum;
 }
@@ -980,6 +983,7 @@ int procInfo2(numout*cc,int nsub,char**name,REAL*mass)
    for(i=0;i<nModelParticles;i++)
    { for(j=0;j<2;j++) decayTable[i].pdList[j]=NULL;
      decayTable[i].width=0;
+     decayTable[i].status=0;
    }
  }
 
