@@ -273,7 +273,7 @@ static int momImproveS21(double *p,double M)
   e1=sqrt(pcm*pcm+m1*m1);
   e2=sqrt(pcm*pcm+m2*m2);
   p[2]   = pcm*cosh(Y)+e1*sinh(Y);
-  p[3]   = pcm*sinh(Y)+e1*cos(Y);
+  p[3]   = pcm*sinh(Y)+e1*cosh(Y);
   p[2+5] =-pcm*cosh(Y)+e2*sinh(Y);                 
   p[3+5] =-pcm*sinh(Y)+e2*cosh(Y);
   p[2+10]= sinh(Y)*M;                 
