@@ -326,17 +326,17 @@ int vegas_int(vegasGrid * vegPtr, long ncall0, double alph,
         {
             xo = DD(j,0);
             xn = DD(j,1);
-//            DD(j,0) = (xo + xn) / 2;
+                    DD(j,0) = (xo + xn) / 2;
             dt[j] = DD(j,0);
             for (i = 1; i < ndm; ++i)
             {
-//                DD(j,i) = xo + xn;
+                    DD(j,i) = xo + xn;
                 xo = xn;
                 xn = DD(j,i+1);
-//                DD(j,i) = (DD(j,i) + xn) / 3;
+                    DD(j,i) = (DD(j,i) + xn) / 3;
                 dt[j] += DD(j,i);
             }
-//            DD(j,ndm) = (xn + xo) / 2;
+                    DD(j,ndm) = (xn + xo) / 2;
             dt[j] += DD(j,ndm);
         }
         for (j = 0; j < dim; ++j)
@@ -346,8 +346,10 @@ int vegas_int(vegasGrid * vegPtr, long ncall0, double alph,
     	      r[i] = 0;
     	      if (DD(j,i) > 0)
     	      {  double xoln = log(dt[j]/DD(j,i));
+    	         if(xoln<0.01)      r[i]=1; else
     	         if (xoln <= 70.f)  r[i] = pow( (1 - exp(-xoln))/xoln, alph);
     	         else               r[i] = pow(  1/xoln,               alph);
+
     	      }
     	      rc += r[i];  
     	   }
