@@ -2,14 +2,12 @@
  Copyright (C) 1997, Alexander Pukhov, e-mail pukhov@theory.npi.msu.su 
 */
 #include"chep_crt.h"
-#include "tptcmac.h"
 #include "syst2.h"
 #include "physics.h"
 #include "s_files.h"
-#include "os.h"
 #include "sortarr.h"
 #include "process.h"
-
+#include <unistd.h>
 
 
 #include "constr.h"
@@ -181,7 +179,7 @@ static int  testin(int l,decayDiagram restmp)
    {
       np = restmp[i-1];
       if (np > 0)
-      {  if (pseudop(np)) return FALSE;
+      {  if (pseudop(np)) return 0;
          j = 1;
          while (inclp[j-1].who != 0 && inclp[j-1].who != np) j++;
          if (inclp[j-1].who == 0 || copyincl[j-1] == 0)
@@ -198,12 +196,17 @@ static int  testin(int l,decayDiagram restmp)
       {
          np = -np;
          if (np > prtclbase[np-1].anti) np = prtclbase[np-1].anti;
-         j = 1;
-         while (liminsp[j-1].who != 0 && liminsp[j-1].who != np) j++;
-         if (liminsp[j-1].who != 0)
-            if (copyins[j-1] == 1) return FALSE; else copyins[j-1]--;
+	 for(j=0;liminsp[j].who;j++)
+	 {
+           if (liminsp[j].who == np)
+           { if(copyins[j]>0) { if (copyins[j] == 1) return 0; else copyins[j]--;}
+	     else             { if (copyins[j]<-1) copyins[j]++;} 
+	   }  
+	 }
       }
    }
+   if(l==ndecay) {for(j=0;liminsp[j].who;j++) if(copyins[j]<-1) return 0;}
+   
    return (nneed <= 0);
 }
 
@@ -294,7 +297,7 @@ static void dooutres(decayDiagram res)
       {
          if (res[k-1] == n_two)
          {
-            lvcpy(copyres,res);
+            memcpy(copyres,res,sizeof(copyres));
             conon(copyres,k);
             if (m != 0) for (l = 1; l <= m; l++)
             {
@@ -306,7 +309,7 @@ static void dooutres(decayDiagram res)
             n_diagram++;
             FWRITE1(copyres,bufres);
             m++;
-            lvcpy(tmplist[m-1],copyres);
+            memcpy(tmplist[m-1],copyres,sizeof(*tmplist));
             label_1: ;
          }
       }
@@ -327,14 +330,13 @@ static void dtc(int* incond,int l,decayDiagram restmp)
    *incond = testin(l,restmp);
    if (!(*incond) || l < ndecay) return;
 
-   *incond = FALSE;
+   *incond = 0;
    res[0] = -nprimary;
    for (i = 1; i <= 2 * (ndecay - 1); i++) res[i] = restmp[i-1];
    dooutres(res);
 	goto_xy(1,24);  print("%u",n_diagram); refresh_scr();
    if (n_diagram > m_diagram)
-       if ( mess_y_n(35,15,"Continue")) m_diagram += 500;
-      else  errorcode = -1;
+      { if ( mess_y_n(35,15,"Continue")) m_diagram += 500;  else  errorcode = -1;}
 }
 
 
@@ -362,7 +364,7 @@ static void decay(int n_part, int l)
        ref[n_part-1][0] = first;
       return;
    }
-   existence = FALSE;
+   existence = 0;
    pp=prtclbase[n_part-1].top;
 
    while (pp != NULL)
@@ -454,14 +456,14 @@ static void decay(int n_part, int l)
                         else 
                         { 
                            ref[n_part-1][l-1] = newelement; 
-                           existence = TRUE; 
+                           existence = 1; 
                         }
                         oldelement = newelement; 
                      } 
-                     equalcond = el1 == el2 ? TRUE : FALSE; 
+                     equalcond = el1 == el2 ? 1 : 0; 
                      el1 = el1->next; 
                   }  while (!(el1 == NULL || equalcond)); 
-                  equalcond = el3 == el2 ? TRUE : FALSE; 
+                  equalcond = el3 == el2 ? 1 : 0; 
                   el2 = el2->next; 
                }  while (!(el2 == NULL || equalcond)); 
                   if (i3 != 0) el3 = el3->next; 
@@ -494,7 +496,7 @@ static void doindex(void)
    old = head;  
    while(FREAD1(res,bufres)==1)   
    {  
-      switch_ =(nin==1); /*  hadr2.how == 0 ? TRUE : FALSE; */
+      switch_ =(nin==1); /*  hadr2.how == 0 ? 1 : 0; */
       next = (indexlink)m_alloc(sizeof(indexStruct));
       old->ilink = next; 
       old = next; 
@@ -504,13 +506,13 @@ static void doindex(void)
       do 
       {   
          if (res[i-1] > 0) 
-         if (switch_) 
-         { 
-            addprtcl(p_list,res[i-1]); 
-            j++; 
-         } 
-         else 
-            switch_ = TRUE; 
+         {  if (switch_) 
+            { 
+              addprtcl(p_list,res[i-1]); 
+              j++; 
+            } 
+            else  switch_ = 1;
+         }    
          i++; 
       }  while (j <= nout); 
 
@@ -671,7 +673,7 @@ static void addbuf(void)
       for (i = 0; i < n_x; i++)
       if(next->outlist[i] != mem[i])  
       {  if(wrtcode)
-         {  lvcpy(outprtcls,inclp); 
+         {  memcpy(outprtcls,inclp,sizeof(outprtcls)); 
             for (j = 0; j < n_x; j++) addprtcl(outprtcls,mem[j]); 
             addrecordtomenu();
          } 
@@ -697,7 +699,7 @@ static void addbuf(void)
       free(old); 
    }
    if(wrtcode)  
-   { lvcpy(outprtcls,inclp); 
+   { memcpy(outprtcls,inclp,sizeof(outprtcls)); 
      for (j = 0; j < n_x; j++) addprtcl(outprtcls,mem[j]); 
      addrecordtomenu();
    } 
@@ -713,7 +715,6 @@ static int* prclist(long * power )
   int * list;
 
   for(i=0;i<l;i++) N*=hadrons[i].pow;
-
   list=malloc(l*N*sizeof(int));
    
   for(i=0;i<l;i++) num[i]=0;
@@ -789,11 +790,11 @@ static int* prclist(long * power )
 
 int construct(void)
 {
-   int          i,j,ndiagram;
+   int          i,j,ndiagram,l;
    char buf_name[STRSIZ];
    int *list;
    long N,m;
-   int l;
+   char format[2]={62,71};
 
    errorcode=0;   
    firstrec=0;
@@ -803,7 +804,7 @@ int construct(void)
    menup=fopen(MENUP_NAME,"wb");
    m_diagram = 500;
    kmenu = 0;
-   f_write("\055\066",2,1,menup);
+   f_write(format,2,1,menup);
    head = (indexlink)m_alloc(sizeof(indexStruct));
    first = (elementlink)m_alloc(sizeof(element)); first->next = NULL;
    zero = (elementlink)m_alloc(sizeof(element));  zero->next = NULL;
@@ -843,6 +844,7 @@ int construct(void)
    }
    free(list);
    cleanProcList();
+   n_diagram=(ftell(diagrp))/sizeof(decayDiagram);
    fclose(diagrp);
 
    
@@ -853,16 +855,8 @@ int construct(void)
    free(first);
    free(zero);
    free(ref);
-   if (n_diagram == 0)
-   {  
-      messanykey(5,22,"Processes of this type are absent");
-      errorcode = -1;
-   }
-   else
-   {
-      be_be();
-      errorcode = 0;
-   }
-   for (i = 17; i <= 24; i++) { goto_xy(1,i); clr_eol(); }   
+   if (n_diagram == 0) errorcode = -1; else errorcode = 0;
+   
+   for (i = 18; i <= 24; i++) { goto_xy(1,i); clr_eol(); }   
    return errorcode;
 }
