@@ -6,7 +6,7 @@
 
 typedef struct catrec
    {
-      int      nsub_, ndiagr_;
+      int      nsub_, ndiagr_,nFile;
       long      factpos, rnumpos, denompos;
    }      catrec;
 
@@ -17,15 +17,16 @@ typedef struct catrec
 
     extern FILE * archiv;
     extern FILE * catalog;
-    extern char mdFls[4][10];
-    extern shortstr  _pathtouser, pathtouser;
+     
+    extern char* mdFls[5];
+    extern shortstr  pathtouser;
     
-#define MENUQ_NAME   scat("%stmp%cmenuq.ch",pathtouser,f_slash)    
-#define MENUP_NAME   scat("%stmp%cmenup.ch",pathtouser,f_slash)
-#define DIAGRP_NAME  scat("%stmp%cproces.tp",pathtouser,f_slash)
-#define DIAGRQ_NAME  scat("%stmp%ccsproces.tp",pathtouser,f_slash)
-#define ARCHIV_NAME  scat("%stmp%carchive.bt",pathtouser,f_slash)
-#define CATALOG_NAME scat("%stmp%ccatalog.tp",pathtouser,f_slash)
+#define MENUQ_NAME   "./tmp/menuq.ch"  
+#define MENUP_NAME   "./tmp/menup.ch"               
+#define DIAGRP_NAME  "./tmp/proces.tp"
+#define DIAGRQ_NAME  "./tmp/csproces.tp"
+#define ARCHIV_NAME  "./tmp/archive_%d.bt"
+#define CATALOG_NAME "./tmp/catalog.tp"
 
 
 extern void  wrt_menu(int menutype, int k,
@@ -33,7 +34,10 @@ extern void  wrt_menu(int menutype, int k,
 extern int  rd_menu(int   menutype, int k,
               char*txt,int*ndel,int*ncalc,int*nrest,long*recpos);
 
+extern int whichArchive(int nFile, int rw);
 #define  FREAD1(d,f)   fread(&(d),sizeof(d),1,f)
 #define  FWRITE1(d,f)  f_write(&(d),sizeof(d),1,f)
+#define  MAXARCHIVE 100000000L
+
 
 #endif
